#!/usr/local/bin/python3
#  a simple CW dashboard maker for key Elemental MediaLive metrics
#  NOT official AWS product in any way!  Just a helper script.
#  R.Clements 04-2021
#  Added the tag and single channel capability
#
import os
import json
from json import JSONEncoder
import code
import time
import copy
from time import gmtime, strftime
import datetime
from datetime import datetime
import subprocess 
#####
myversion = "13"
#####
# ------------------------------------------------------------------------------------
## Definitions of various metrics: 
## Dropped Frames: The Dropped Frame metric indicates the cumulative number of dropped input frames since the start of the channel. A frame drop indicates that the system is unable to keep up in real time.
## Input Video Frame Rate: The input video frame rate averaged over the last ten seconds.
## Network In: A graph of the sum of the input bit rate.  Big fluctuations may indicate an inbound circuit issue.
## Fill Milliseconds: This metric is the number of consecutive milliseconds of "fill" frames (e.g. repeated frames, black frames and/or slate) inserted by the MediaLive due to the input not being present.  Once the input returns it will go back to zero.
## SVQ Time: This metric defines the percentage of time, averaged over the last 10 seconds, the encoder has had to reduce quality optimizations to speed up the encode process, in order to keep the MediaLive running in real time.

## ---------------------------------------------------------------------------
## Individual metrics dictionaries follow.
## NetworkIn widget dictionary template

EML_NetworkIn_widget_d = {
            "type": "metric",
            "x": 0,
            "y": 0,
            "width": 21,
            "height": 6,
            "properties": {
                "metrics": [
                    [ "MediaLive", "NetworkIn", "ChannelId", "firstchannelID", "Pipeline", "0", { "label": "firstchannelID p0" } ]
                ],
                "view": "timeSeries",
                "stacked": "false",
                "title": "MediaLive - AVG NetworkIn Bytes [rolling avg]",
                "region": "us-west-2",
                "period": 60,
                "stat": "Average",
                "yAxis": {
                    "left": {
                        "min": 0
                    }
                },
                "legend": {
                    "position": "right"
                }
            }
}
#
##-------------------------------------------------------------------------------------------------------
### FillMsec widget dictionary template
EML_Fillframes_widget_d = {
            "type": "metric",
            "x": 0,
            "y": 6,
            "width": 21,
            "height": 6,
            "properties": {
                "metrics": [
                    [ "MediaLive", "FillMsec", "ChannelId","firstchannelID", "Pipeline", "0", { "label": "firstchannelID p0" } ]
                ],
                "view": "timeSeries",
                "stacked": "false",
                "region": "us-west-2",
                "title": "Ms of Fill frames inserted on outputs [rolling avg] - indicates possible input issue",
                "stat": "Average",
                "period": 60,
                "legend": {
                    "position": "right"
                }
            }
}
##-------------------------------------------------------------------------------------------------------
###  SVQ widget dictionary template
EML_SVQ_widget_d = {
        	"type": "metric",
            "x": 0,
            "y": 9,
            "width": 21,
            "height": 6,
            "properties": {
                "metrics": [
                    [ "MediaLive", "SvqTime", "ChannelId", "firstchannelID", "Pipeline", "0", { "label": "firstchannelID p0" } ]
                ],
                "view": "timeSeries",
                "region": "us-west-2",
                "title": "Quality Reductions to maintain framerate [rolling avg] ",
                "period": 60,
                "liveData": "true",
                "stacked": "false",
                "yAxis": {
                    "left": {
                        "min": 0,
                        "max": 10,
                        "label": "Forced Quality Reductions"
                    }
                },
                "stat": "Average",
                "legend": {
                    "position": "right"
                }
	}
}
##-------------------------------------------------------------------------------------------------------
EML_InputFrmRt_widget_d = {
            "type": "metric",
            "x": 0,
            "y": 12,
            "width": 21,
            "height": 6,
            "properties": {
                "metrics": [
                    [ "MediaLive", "InputVideoFrameRate", "ChannelId", "firstchannelID", "Pipeline", "0", { "label": "firstchannelID p0" } ]
                ],
                "view": "timeSeries",
                "stacked": "false",
                "region": "us-west-2",
                "period": 60,
                "yAxis": {
                    "left": {
                        "min": 0,
                        "max": 75,
                        "label": "FPS"
                    }
                },
                "stat": "Average",
                "title": "Input Frame Rate [rolling avg]",
                "legend": {
                    "position": "right"
                }
            }
}
#-------------------------------------------------------------------------------------------------------
RTP_widget_d = {
           "type": "metric",
            "x": 0,
            "y": 15,
            "width": 21,
            "height": 6,
            "properties": {
    "metrics": [
        [ "MediaLive", "RtpPacketsReceived", "ChannelId", "firstchannelID", "Pipeline", "0", { "label": "Pkts-Received_chanID_PL0" } ],
        [ ".", "RtpPacketsRecoveredViaFec", ".", ".", ".", ".", { "label": "Pkts-RecoveredViaFec_chanID_PL0" } ],
        [ ".", "RtpPacketsLost", ".", ".", ".", ".", { "label": "Pkts-Lost_chanID_PL0" } ]
    ],
    "view": "singleValue",
    "stacked": "false",
    "region": "us-west-2",
    "title": "RTP Packet Status",
    "period": 60,
    "stat": "Sum",
    "title": "RTP Input Metrics"
  },
  "legend": {
                    "position": "right"
                }
}

#-----------------------------------------------------------------------------------------------------
## Aggregate all the metrics dictionaries into a list of dictionaries
WidgetsList = [EML_NetworkIn_widget_d, EML_InputFrmRt_widget_d,EML_Fillframes_widget_d,EML_SVQ_widget_d]
## Create an empty list of channels which need separate RTP widgets
RTPWidgetsList=[]
#-----------------------------------------------------------------------------------------------------
def GetCLIResult(mycmd):
        try:
                result = subprocess.run(mycmd, stdout=subprocess.PIPE)
                DecodedReply = json.loads(result.stdout.decode('utf-8'))
                return(DecodedReply)
        except:
                print("Could not get valid CLI resonse to cmd: ",mycmd, "...Please report this error to the author. Exiting.")
                #code.interact(local=globals())
                exit()
#-------------------------------------------------------------------------------------------------------
# compute timestamps for files
tnow = datetime.now()
tstamp = tnow.strftime("%m%d%y_%H%M%S")
#-------------------------------------------------------------------------------------
quote='"'
print("\n","========== WELCOME TO THE EASY DASHBOARD MAKER v",myversion," ============")
print("   This script makes a Dash for:  MediaLive Input Monitoring","\n\n")
print("\n","\n",)
Answers = ['a','t','s']
RunMode = input("--> This script can include [a]ll channels, [t]agged channels, or a [s]ingle channel. Please choose a,t,s ? (any other char quits) " )
if RunMode not in Answers:
	print("----> exiting.") 
	exit()
	
##--------------------------------------------------------------------------------------
## Poll for all channels before we parse down the List
AllCannelsJSON_d = {'Channels': [{}]}  ## empty dictionaries for all channels and target channels
TargetChannelsJSON_d= {'Channels': []}
##
cmd = ['/usr/local/bin/aws','--no-paginate','medialive','list-channels']
try:
	AllCannelsJSON_d = GetCLIResult(cmd)
	## AllCannelsJSON is the list of all the channels we found. We will cull this list as needed
except:
	print("!! No valid reply from list-channels call. Sorry!!  --> Exiting .")
	exit()

ChCount = len(AllCannelsJSON_d['Channels'])
if (ChCount == 0 ):
	print("---> No Channels found for this Account in this region; exiting.")
	exit()	
		
##--------------------------------------------------------------------------------------
if RunMode == 't' :
	MyProjectName = input(" -> Please enter the value of the ProjectName to use when selecting Channels for this Dashboard: " )
	if (MyProjectName == ""): 
		print("  --> This option requires a value for the Tag called ProjectName. Exiting.")
		exit()
	
	for ChIndex in range (0, ChCount):  ## now we iterate all the chx matching tags
		ChPointer = AllCannelsJSON_d['Channels'][ChIndex]
		if ('Tags' in ChPointer) and ( 'ProjectName' in ChPointer['Tags']) and (ChPointer['Tags']['ProjectName'] == MyProjectName):
			print("----> Found matching channel ",ChPointer['Name'])
			TLL = len(TargetChannelsJSON_d['Channels'])
			#code.interact(local=globals())
			TargetChannelsJSON_d['Channels'].append(ChPointer)
			
		else:
			print(" ...")
	
	#code.interact(local=globals())
##------------------------------------------------------------------------------------
if RunMode == 's' :
	# display all channels and prompt for desired channel ID
	
	print("\n\n","  This account has the following channels defined:")	
	print("  ","NAME","                                ","ID")
	for ChIndex in range (0, ChCount):  ## now we list all the chx
		chName = (AllCannelsJSON_d['Channels'][ChIndex]['Name'])
		chName = chName.ljust(32, ' ')
		chanID = (AllCannelsJSON_d['Channels'][ChIndex]['Id'])
		print("  ",chName, "    ",chanID)
	
	print("\n")
	SingleCh = input("--> Specify (copy+paste) the Channel ID to use: ")
	
	for ChIndex in range (0, ChCount):  ## now we iterate all the chx matching by ID
		ChPointer = AllCannelsJSON_d['Channels'][ChIndex]
		if (SingleCh == ChPointer['Id'] ):
			print("----> Adding single channel",ChPointer['Name'])
			#code.interact(local=globals())
			TargetChannelsJSON_d['Channels'].append(ChPointer)
			break
		else:
			print(" ...")
	
	#code.interact(local=globals())
##------------------------------------------------------------------------------------
if RunMode == 'a' :
	TargetChannelsJSON_d = copy.deepcopy(AllCannelsJSON_d)
	#code.interact(local=globals())
##------------------------------------------------------------------------------------
print("\n\n","---> Processing", len(TargetChannelsJSON_d['Channels'])," Channel(s) for new Dash...","\n")
##	
##===============================================================================================
## ===  WORK ON TARGET LIST FROM THIS POINT FORWARD  ===========================
##===============================================================================================
# Set region and compare later
myregion = "us-east-1"
ChCount = len(TargetChannelsJSON_d['Channels'])
MetricTemplateLine = [ "...", "nextchannelID", "Pipeline", "nextpipevalue", { "label": "labelvalue" } ]
quote='"'
TotalPipelines = -1
# ------------------------------------------------------------------------------------
## Iterate through all the discovered channels
# set a boolean for need RTP widget
NeedsRTPWidget = 0
TotalRTPWidgets=0
#
for ChIndex in range (0, ChCount):
	chName = ""
	MetricTemplateLine = [ "...", "nextchannelID", "Pipeline", "nextpipevalue", { "label": "labelvalue" } ]
	chName = (TargetChannelsJSON_d['Channels'][ChIndex]['Name'])
	chanID = (TargetChannelsJSON_d['Channels'][ChIndex]['Id'])
	chName = chName.ljust(32, ' ')
	print("\n","--> Scanning Channel ",chanID)
	cmd = ['/usr/local/bin/aws','--no-paginate','medialive','describe-channel','--channel-id', chanID]
	try:
		CLIReply = GetCLIResult(cmd)
        #print("DecodedReply has heys", CLIReply.keys())
	except:
		print("!! No keys found in CLIReply for cmd: ",cmd)
		print("!! Exiting.")
		exit()

	ThisChanneljson = CLIReply
	
	# Now check the region again
	arnstring=str(ThisChanneljson['Arn'])
	arnlist=arnstring.split(":",9)
	if (myregion != arnlist[3]) :
		myregion = arnlist[3]
		print("-----> Setting region to ",myregion," but you can change it later if needed.")
		
	TotalPipelines = (TotalPipelines + 1)
	
	#print("Total Pipelines added = ",TotalPipelines)
	# handle case where channel has RTP inputs separately 
	# have to check every input on this channel...sigh...
	for inputcounter in range (0,len(ThisChanneljson['InputAttachments'])) :
		
		ThisInputID = ThisChanneljson['InputAttachments'][inputcounter]['InputId']
		ThisInputName = ThisChanneljson['InputAttachments'][inputcounter]['InputAttachmentName']
		print("-------> Found input",ThisInputName)
		## query the input via cli 
		cmd = ['/usr/local/bin/aws','--no-paginate','medialive','describe-input','--input-id', ThisInputID]
		ThisInputJSON = GetCLIResult(cmd)
			
		if ThisInputJSON['Type']  == "RTP_PUSH" :
			NeedsRTPWidget = 1
			print("-------> Found Channel with RTP inputs, adding lines to RTP widget...")
			## add an empty element to the RTP widget list
			TL = ['']
			TL[0] = {
			   "type": "metric",
				"x": 0,
				"y": 15,
				"width": 9,
				"height": 3,
				"properties": {
				"metrics": [
					[ "MediaLive", "RtpPacketsReceived", "ChannelId", "firstchannelID", "Pipeline", "0", { "label": "." } ],
					[ ".", "RtpPacketsRecoveredViaFec", ".", ".", ".", ".", { "label": "." } ],
					[ ".", "RtpPacketsLost", ".", ".", ".", ".", { "label": "." } ]
				 ],
				"view": "singleValue",
				"stacked": "false",
				"region": "us-west-2",
				"title": "RTP Packet Status",
				"period": 3600,
				"stat": "Sum",
				"title": "RTP Input Metrics [Total for last hr]"
					}
				}
			#print("\n ! 243 RTPWidgetsList has",len(RTPWidgetsList),"elements CF0 and now reads:", RTPWidgetsList,"\n\n")
			#print("\n\n")
			# first line of RTP metric
			TL[0]['properties']['metrics'][0][3] = chanID
			TL[0]['properties']['metrics'][0][5] = "0"
			TL[0]['properties']['metrics'][0][6]['label'] = "Pkts-Recvd_CH:"+chanID+"_PL0"
			## second line of RTP Metric
			TL[0]['properties']['metrics'][1][3] = chanID
			TL[0]['properties']['metrics'][1][5] = "0"
			TL[0]['properties']['metrics'][1][6]['label'] = "Pkts-RecoveredViaFec_CH:"+chanID+"_PL0" 
			## third line of RTP metric
			TL[0]['properties']['metrics'][2][3] = chanID
			TL[0]['properties']['metrics'][2][5] = "0"
			TL[0]['properties']['metrics'][2][6]['label'] = "Pkts-Lost_CH:"+chanID+"_PL0" 
			## title of Metric
			Q = "RTP Packet Status for Channel:"+chanID+", Input:"+ThisInputName
			TL[0]['properties']['title'] = Q
			## TL is now populated
			#print("\n\n\n","at 281 TL is now:", TL,"\n\n")
			##	
			#print("\n ! 283 RTPWidgetsList has",len(RTPWidgetsList),"elements CF0 and now reads:", RTPWidgetsList,"\n\n")	
			#code.interact(local=globals())
			#RTPWidgetsList[TotalRTPWidgets]=TL[0]
			RTPWidgetsList.append(TL[0])
			TotalRTPWidgets+=1
			
		#
	# handle pipeline0 first
	thislabel = "CH:"+chanID + "_PL:0"
	if (TotalPipelines == 0) :
		# special case for first line of each metric due to param counts
		counter = 0
		
		for counter in range ( 0, len(WidgetsList) ) :
			try:
				WidgetsList[counter]['properties']['metrics'][0][3] = chanID
				WidgetsList[counter]['properties']['metrics'][0][5] = "0"
				WidgetsList[counter]['properties']['metrics'][0][6]['label'] = thislabel
				WidgetsList[counter]['properties']['region']= myregion
			except:
				print("unable to add row 0 the widgets for Ch ",chanID, "...continuing..." )
				
	
	else:
		## we are in rows 2-N, so add a line to each metric: 
		#print("\n","Adding a row to each widget for channel",chanID)
		for counter in range (0,len(WidgetsList)) :
			WidgetsList[counter]['properties']['metrics'].append(MetricTemplateLine)
		## now set the values		
		try:
			for counter in range (0,len(WidgetsList)) :
				print("---------> Adding another row to widgets...")
				WidgetsList[counter]['properties']['metrics'][TotalPipelines][1] = chanID
				WidgetsList[counter]['properties']['metrics'][TotalPipelines][3] = "0"
				WidgetsList[counter]['properties']['metrics'][TotalPipelines][4]['label'] = thislabel
		except:
			print("unable to add a row to the widgets for Ch ",chanID, "...continuing..." )

		# determine if this channel has a second pipeline and if so add p1 to the metrics
	if (ThisChanneljson['ChannelClass'] == 'STANDARD') :
		#print("-----> found STANDARD pipeline on Ch",chanID, "...configuring widget for both piplines...")
		##
		TotalPipelines = (TotalPipelines + 1)
		MetricTemplateLine = [ "...", "nextchannelID", "Pipeline", "nextpipevalue", { "label": "labelvalue" } ]
		thislabel = "CH:"+chanID + "_PL:1"
		for counter in range (0,len(WidgetsList)) :
			WidgetsList[counter]['properties']['metrics'].append(MetricTemplateLine)
		#
		try:
			for counter in range (0,len(WidgetsList)) :
				WidgetsList[counter]['properties']['metrics'][TotalPipelines][1] = chanID
				WidgetsList[counter]['properties']['metrics'][TotalPipelines][3] = "0"
				WidgetsList[counter]['properties']['metrics'][TotalPipelines][4]['label'] = thislabel

		except:
			print("\n\n","---> Could not add pipeline 1 metrics for ",chanID, "...but you can add them manually. Continuing...")
		
	else:
		## single pipeline
		print("")

# ------------------------------------------------------------------------------------

## now we assemble the complete json 

FileOpen = '{"widgets": ['
FileClose = ']}'
## 
##  Start Assembling full JSON as a list of strings 
WidgetStringsList=[]
#
if (NeedsRTPWidget) :
	#code.interact(local=globals())
	for J in range (0, len(RTPWidgetsList)):
		WidgetsList.append(RTPWidgetsList[J])
#
FullJSON = FileOpen
for counter in range (0, len(WidgetsList)) :
	S = str(WidgetsList[counter])
	S = S.replace('(','')
	S = S.replace(')','')
	S = S.replace("'",'"')
	S = S.replace('"false"','false')
	S = S.replace('"true"','true')
	S+=","
	FullJSON+=S
##
##remove the comma after final widget and finish the JSON
FullJSON = FullJSON[:-1]
FullJSON+=FileClose
##
print("")
dashfile = "MediaLive_dashjson_" + tstamp
f = open(dashfile, 'w')
f.write(FullJSON) 
f.close()
tyellow="\033[1;33;40m"
tnormal="\033[0;37;40m"
print(tnormal,"\n")

print("----> Dash JSON exported to local file '", dashfile, "' for future reference." )    
##
### now create the dashboard 

dashname = "MediaLive_Inputs_Dash_" + tstamp
dashcmd = "aws cloudwatch put-dashboard --dashboard-name "+ dashname +" --dashboard-body file://" + dashfile
print("\n","=================================================================","\n")
print(" Attempting to create new dashboard ",tyellow, dashname, tnormal, "in your CW Console. You can rename the dash via the Console.")
dashresult = os.popen(dashcmd).read()
print("\n\n"," Cloudwatch upload result was: (empty [ ] means no errors) ","\n",dashresult)
#code.interact(local=globals())
#print("Try running cmd: ", dashcmd)
print(" Remember to set the time range for your new Dashboard (upper right corner of CW Console).")

exit()
